#include "Minuit2/Minuit2Minimizer.h"
#include "Math/Functor.h"
#include "Likelihood_FCN.hh"
#include <iostream>
#include <string>
#include "TH3.h"
#include "TF1.h"
#include "TStyle.h"
#include "TList.h"
#include "TCanvas.h"
#include "Math/Minimizer.h"
#include "Math/Factory.h"
#include "Math/Functor.h"
#include "DetectorLayout.hh"

double my_transfer_function(const double *x, const double*)
{
  if(*x < 3.) return 0;
  if(*x < 26.) return 0.1;
  if(*x < 78.) return 0.3;
  return 0.6;
}
int MLEM()
{
  ROOT::Math::Minimizer* minim = ROOT::Math::Factory::CreateMinimizer("Minuit2","Combined");
  minim->SetMaxFunctionCalls(10000);
  minim->SetMaxIterations(10000);
  minim->SetTolerance(0.1);
  minim->SetPrintLevel(3);
  Likelihood_FCN lf = Likelihood_FCN("PoCA_Result.root","ML_Info","PoCA");
  int nvariable = lf.GetNVariable();
  ROOT::Math::Functor f(lf, nvariable);
  double *init_value = lf.Init();
  lf(init_value);
  minim->SetFunction(f);
  for(int i=0;i<nvariable;i++)
  {
    const string name = "x" + std::to_string(i);
    minim->SetLowerLimitedVariable(i,name,init_value[i],0.1,2.3);
  }
  minim->Minimize();
  const double *xs = minim->X();
  int NBin_X, NBin_Y, NBin_Z;
  NBin_X = lf.NBin_X;
  NBin_Y = lf.NBin_Y;
  NBin_Z = lf.NBin_Z;
  double HalfX = Plate_Size_X/20; 
  double HalfY = Plate_Size_Y/20; 
  double HalfZ = ObjectAreaWidth/20;
  TH3F *h3 = new TH3F("Muon Image","MLS Image",NBin_X, -HalfX, HalfX, NBin_Y,
                     -HalfY, HalfY, NBin_Z, -HalfZ, HalfZ);
  for(int i=0; i < nvariable; i++)
  {
    int nz = i/(NBin_X*NBin_Y);
    int ny = (i-nz*NBin_X*NBin_Y)/NBin_X;
    int nx = i-nz*NBin_X*NBin_Y-ny*NBin_X;
    std::cout << nx <<", "<< ny << ", "<<nz<<", "<<xs[i]<<std::endl;
    h3->SetBinContent(nx+1, ny+1, nz+1, xs[i]);
  } 
  // Plot
  gStyle->SetCanvasPreferGL(1);
  TCanvas *c = new TCanvas("glc","MuImag",0,0,HalfX*40,HalfZ*40);
  TList *flist = h3->GetListOfFunctions();
  if(flist)
  {
    TF1 *tf = new TF1("TransferFunction", my_transfer_function);
    flist->Add(tf);
  }
  gStyle->SetPalette(1);
  c->Divide(2,2);
  c->cd(1);
  TH2F *h_zx = (TH2F*) h3->Project3D("zx");
  h_zx->SetStats(0);
  h_zx->Draw();
  c->cd(2);
  h3->Project3D("zy")->SetStats(0);
  h3->Project3D("zy")->Draw();
  c->cd(3);
  h3->Project3D("yx")->SetStats(0);
  h3->Project3D("yx")->Draw();
  c->cd(4);
  h3->Draw("glcolz");
  std::cout <<"done" << std::endl;

  return 0;  
}
